(* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: gonttree.mli,v 1.9 2002/05/27 15:08:33 malekith Exp $
 *
 * Module interface: Gonttree
 * Description: Various datatypes definitions for Gont compiler. There
 * is no implementation for this module (as there are only types here).
 *)

(* Identifier. *)
type id = string

(* Location in source file. *)
type location = {mutable loc_file: string; 
                 mutable loc_char: int;
                 mutable loc_line: int}

(* Types used by Gont compiler. *)
type ty =
  | Int                         (* integer of the same size as pointer *)
  | Void                        (* unit type *)
  | Bool                        (* boolean, same size as Int *)
  | Float                       (* C's double *)
  | String                      (* Pointer to struct string$ *)
  | Tuple of ty list            (* void ** *)
  | Func of ft_record
  | TyVar of id
  | Struct of field_record list
  | OptStruct of field_record list
  | Union of field_record list
  | AbstractType                (* Used when outside module defines `type t' *)
  | NamedType of nt_record      (* <int>List::t, some_struct *)
  | Exn                         (* Type of exception value. *)
and typedef = 
   {mutable td_tyvars: id list;  (* Parameters of defined type. *)
    mutable td_type: ty;         (* Actual defined type (Struct or Union in
                                  * most cases). *)
    mutable td_module: id;       (* "" for current module, until Gontbind *)
    td_name: id;
    td_loc: location;            (* Where it was defined *)
   } 
and nt_record = 
   {nt_parms: ty list;          (* Actual types to be substituted to
                                 * nt_tydef.ty_vars *)
    nt_module: id;              (* "" if module qualifier not given. *)
    nt_name: id;
    nt_loc: location;           (* Where it was used? We need it, as we might
                                 * want to generate error message about it. *)
    mutable nt_tydef: typedef option; (* Filled by Gontbind. *)
    (* nt_type dropped, use instance_value. *)
   }
(* ft_tyvars is generally filled by the parser using simple_fv function
 * called on parameters and return type. However this is wrong for nested
 * functions (for example `f('a x) { g('a x, 'b y) {} }', ft_tyvars for g
 * should be ['b], but it is ['a, 'b]), this is fixed by Gonttypes module. 
 * That's why we need ft_tyvars to be mutable. *)
and ft_record = {ft_ret_type: ty;
                 ft_parms: ty list;
                 mutable ft_tyvars: id list}
and field_record = {mutable fr_module: id;
                    fr_name: id;
                    mutable fr_sym: symbol option;
                    fr_type: ty;
                    fr_loc: location}

(* Abstract type of symbol. *)
and symbol_property =
  | IsInClosure
  | IsUnionField
  | IsTopPattern
  | IsNonLvalue
  | IsPrototyped
  | IsNonLocal
  | IsInitialized
  | IsInPattern
  | IsGlobalVar
  | IsException
  | IsFormalParm
  | UnionFieldId of int
  | LoopLevel of int
  | ExternName of string

(* One kind for each namespace. *)
and symbol_kind =
  (* This includes global and local variables and functions. Also union
   * fields and exceptions go here. *)
  | VariableKind
  (* Conceptually each structure provides separate namespace, but this 
   * is achieved by maintaining list of structures given symbol belongs. 
   * We need to do global lookups whenever we see StructExp anyway.
   * (as it might have any structure type). *)
  | StructFieldKind
  (* Labels assigned to loops, that can be used by break and continue
   * stmts. *)
  | LoopIdKind

and symbol = 
  {
   (* Kind of symbol. Kind corresponds roughly to namespace. Note that named
    * types are handled by typedef type, not symbol. *)
   mutable sym_kind: symbol_kind;
   (* Name of symbol, Module:: prepended for globals. *)
   mutable sym_name: id;
   (* Type of symbol. *) 
   mutable sym_type: ty;
   (* Level of function nesting. 0 for globals, -1 for symbols from other
    * modules, see gontbind.ml for more info. *)
   mutable sym_fnc_level: int;
   (* Same as above, but s/function/scope/. *)
   mutable sym_scope_level: int;
   (* List of special properties symbol might have. *)
   mutable sym_properties: symbol_property list;
   (* [] for regular symbols, [union_type] for union fields, for struct field 
    * list of typedefs, this field was used in. *)
   mutable sym_typedefs: typedef list;
   }
   
(* Type of binary operator. There is no distinction of Int* and Float* 
 * operators anymore. *)
type op2 =
  | Plus
  | Minus
  | Mult
  | Div
  | Mod
  | Assign
  | Eq | Ne | Lt | Gt | Ge | Le
  
(* Type of unary operators, Int/Float, as above. *)
type op1 =
  | Negate
  | PreDec
  | PreInc
  | PostDec
  | PostInc

(* Formal parameter of function. parm_symbol set by Gontbind. Note that it
 * still can be in closure (it is assigned there at the function startup). *)
type parm_record = {parm_name: id;
                    parm_type: ty;
                    mutable parm_symbol: symbol option}

(* Expression and statement representation trees. *)
type raw_exp =
  | SimpleBinOp of op2 * exp * exp
  | SimpleUnOp of op1 * exp
  | Ref of ref_record   (* reference to variable *)
  | Call of exp * exp list  (* functional variable * parameters *)
  | IntConst of string
  | BoolConst of bool
  (* Integer and Float constants are represented as strings, because
   * we might not be able to support target's integer/float size
   * correctly in Caml (we would have to use Int64 to be reasonably
   * safe, but even this might not be right), and even if, we don't 
   * need the actual value, we only need to pass it to Ksi compiler
   * (and hope, it will interpret it correctly :^) *)
  | FloatConst of string
  | VoidConst (* () in ML *)
  | StringConst of string
  | NullConst                          (* null *)
  | TupleExp of exp list
  | UnionExp of union_record
  | Lambda of lambda_record
  (* Module::name *)
  | CrossModuleRef of xmr_record
  | StructExp of struct_init list      (* {foo=1, bar=x+y} *) 
  | DotExp of de_record                (* foo.bar *)
and exp = {mutable raw_exp: raw_exp;
           (* Filled by Gonttypes, but don't use it directly, call
            * Gonttypes.type_of instead. *)
           mutable exp_type: ty option;
           exp_loc: location}
and ref_record = {
                  (* This should be accessed only by Gontbind, after binding
                   * you should use ref_symbol#get_name (as it includes
                   * module name for global variables, and might be mangled
                   * for locals). *)
                  ref_name: id;
                  (* How deep in closure chain should we go to get to
                   * the value. If ref_level == 0, the value is defined
                   * at the same function level, and we don't need to
                   * go anywhere to get to it :^), however if 
                   * ref_symbol#in_closure, we need to access it through
                   * cl$ pointer. *)
                  mutable ref_level: int;
                  (* Symbol filled by Gontbind. *)
                  mutable ref_symbol: symbol option}
and xmr_record = {
                  (* Same as for ref_record.ref_name -- don't use after 
                   * binding. *)
                  xmr_module: id;
                  xmr_name: id;
                  (* Symbol filled by Gontbind. *)
                  mutable xmr_symbol: symbol option}
and union_record = {
                    (* Actually it is member of union, one record for each
                       member *)
                    ur_exp: exp;
                    ur_module: id;
                    ur_name: id;
                    mutable ur_sym: symbol option}
and struct_init = {
                   (* Structure expression have form: 
                        {module::name1 = e1,
                         name2 = e2, ... } *)
                   si_module: id;
                   si_name: id;
                   mutable si_sym: symbol option;
                   si_expr: exp;
                   si_loc: location}
and de_record = {de_expr: exp;
                 de_id: id;
                 (* Warning: this is set by Gonttypes, not Gontbind. *)
                 mutable de_sym: symbol option}
                  
and raw_stmt =
  | Skip
  | Exp of exp                  (* Expression statement -- evaluate expression
                                 * and forget the value. *)
  | Return of exp               (* Return given value from function. *)
  | ReturnVoid                  (* Hm.. we have currently no way of 
                                 * representing void value... (i.e. '()') *)
  | VarDef of var_record        (* int foo; *)
  | Block of stmt list          (* { ... } *)
  | Pair of stmt * stmt         (* Doesn't introduce new scope, simply
                                 * groups statements, it is introduced
                                 * by few parser rules, however there is
                                 * no user-mode way to get it. *)
  (* Stores base expression which is matched to first fitting pattern
     in (pattern*stmt) list and its stmt is executed *)
  | Switch of exp * (case_clause list)
  | Let of ((pattern * exp) list) * stmt
  | If of exp * stmt * stmt     (* if (exp) stmt else stmt *)
  (* All loops are constructed from these two terms. for(a;b;c)X loop is
   * split into Block(a,Loop(Pair(Pair(BreakIfNot(b),X),c))), while(c)X
   * into Loop(Pair(BreakIfNot(c),X)), and so on. *)
  | Loop of loop_record
  | BreakIfNot of exp
  (* This if for regular, user-supplied, breaks and continues. *)
  | Break of break_record
  (* Local open -- import all symbols from given module into current 
   * namespace, however note, symbols local to module always override
   * external symbols! *)
  | Open of id
  | Try of try_record
  | Raise of exp                       (* raise Foo["bar", baz] *)
and stmt = {raw_stmt: raw_stmt;
            stmt_loc: location}
and loop_record = {ll_label: id;                (* If unlabeled, it's "" *)
                   mutable ll_sym: symbol option;  (* Of kind LoopIdKind *)
                   ll_body: stmt;
                   ll_inc: stmt;        (* s3 in for(s1;s2;s3), executed, 
                                         * even is continue is called. *)
                   }
and break_record = {br_label: id;               (* If unlabeled, it's "" *)
                    mutable br_sym: symbol option;
                    br_continue: bool;          (* Whatever this break is 
                                                 * in fact a continue. *)
                    }
and var_record = {var_name: id;         (* Don't use after binding. *)
                  var_type: ty;
                  (* Filled by Gontbind. *)
                  mutable var_symbol: symbol option}
and lambda_record = {lambda_ret_type: ty;
                     lambda_parms: parm_record list;
                     lambda_body: stmt;
                     lambda_tyvars: id list}    (* Free type variables. *)
and case_clause = {cc_pattern: pattern;
                   cc_stmt: stmt}
and pattern = {mutable raw_pattern: raw_pattern;
               mutable pat_loc: location}
and raw_pattern = 
  | VoidPattern
  | VarPattern of var_pattern_record
  | ExprPattern of exp
  | TuplePattern of tuple_pattern_record
  | UnionPattern of union_pattern_record
and tuple_pattern_record = {tpr_patterns: pattern list}
and union_pattern_record = {upr_module: id;
                            upr_name: id;
                            mutable upr_sym: symbol option;
                            upr_pattern: pattern}
and var_pattern_record = {vp_name: id;
                  mutable vp_symbol: symbol option}

and try_record = {try_body: stmt;
                  try_with: case_clause list;
                  try_finally: stmt}

type fn_decl = {fn_ret_type: ty; 
                fn_name: id; 
                fn_parms: parm_record list;
                fn_body: stmt;
                fn_loc: location;
                fn_tyvars: id list;
                mutable fn_symbol: symbol option;
                fn_export_as: string}

type extern_c_decl = {xc_type: ty; 
                      xc_name: id; 
                      xc_loc: location;
                      mutable xc_symbol: symbol option}

type global_decl = {global_type: ty;
                    global_loc: location;
                    global_name: id;
                    mutable global_symbol: symbol option}

type fn_proto_decl = {fp_type: ty;
                      fp_name: id;
                      fp_loc: location;
                      fp_extern_name: string;
                      mutable fp_symbol: symbol option}

type ed_record = {ed_type: ty;
                  ed_name: id;
                  ed_loc: location;
                  mutable ed_sym: symbol option}
                  
type sd_record = {sd_name: id;
                  sd_body: stmt;
                  sd_loc: location}

type decl =
  | FuncDecl of fn_decl
  | GlobalDecl of global_decl
  | ExternC of extern_c_decl
  | FuncProto of fn_proto_decl
  | TopLevelOpen of id
  | TypeDef of typedef
  | ExceptionDecl of ed_record
  | DummyDecl
  | SectionDecl of sd_record
;;

